import { injectLoader, pick, injectStore } from 'angular-three';
import { assertInjector } from 'ngxtension/assert-injector';
import { FBXLoader, FontLoader, DRACOLoader, MeshoptDecoder, GLTFLoader } from 'three-stdlib';
import * as i0 from '@angular/core';
import { signal, effect, computed, inject, ChangeDetectorRef, input, viewChild, untracked, Component, ChangeDetectionStrategy } from '@angular/core';
import { mergeInputs } from 'ngxtension/inject-inputs';
import { DefaultLoadingManager, TextureLoader, Texture } from 'three';

function _injectFBX(input, { injector } = {}) {
    return assertInjector(_injectFBX, injector, () => {
        return injectLoader(() => FBXLoader, input);
    });
}
_injectFBX.preload = (input) => {
    injectLoader.preload(() => FBXLoader, input);
};
const injectFBX = _injectFBX;

let fontLoader = null;
async function loadFontData(font) {
    return typeof font === 'string' ? await (await fetch(font)).json() : font;
}
function parseFontData(fontData) {
    if (!fontLoader) {
        fontLoader = new FontLoader();
    }
    return fontLoader.parse(fontData);
}
const cache = new Map();
function injectFont(input, { injector } = {}) {
    return assertInjector(injectFont, injector, () => {
        const font = signal(null);
        effect(() => {
            const fontInput = input();
            if (cache.has(fontInput)) {
                font.set(cache.get(fontInput));
                return;
            }
            loadFontData(input()).then((data) => {
                const parsed = parseFontData(data);
                cache.set(fontInput, parsed);
                font.set(parsed);
            });
        });
        return font.asReadonly();
    });
}
injectFont.preload = (input) => {
    loadFontData(input()).then((data) => {
        const parsed = parseFontData(data);
        cache.set(input(), parsed);
    });
};
injectFont.clear = (input) => {
    if (input) {
        cache.delete(input());
    }
    else {
        cache.clear();
    }
};

let dracoLoader = null;
let decoderPath = 'https://www.gstatic.com/draco/versioned/decoders/1.5.5/';
function _extensions(useDraco, useMeshOpt, extensions) {
    return (loader) => {
        if (extensions) {
            extensions(loader);
        }
        if (useDraco) {
            if (!dracoLoader) {
                dracoLoader = new DRACOLoader();
            }
            dracoLoader.setDecoderPath(typeof useDraco === 'string' ? useDraco : decoderPath);
            loader.setDRACOLoader(dracoLoader);
        }
        if (useMeshOpt) {
            loader.setMeshoptDecoder(typeof MeshoptDecoder === 'function' ? MeshoptDecoder() : MeshoptDecoder);
        }
    };
}
function _injectGLTF(path, { useDraco = true, useMeshOpt = true, injector, extensions, onLoad, } = {}) {
    return assertInjector(_injectGLTF, injector, () => {
        const result = injectLoader(() => GLTFLoader, path, {
            extensions: _extensions(useDraco, useMeshOpt, extensions),
            // @ts-expect-error - we know the type of the data
            onLoad,
        });
        Object.defineProperty(result, 'scene', {
            value: computed(() => {
                const gltf = result();
                if (!gltf)
                    return null;
                return gltf.scene;
            }),
        });
        return result;
    });
}
_injectGLTF.preload = (path, { useDraco = true, useMeshOpt = true, extensions, onLoad, } = {}) => {
    injectLoader.preload(() => GLTFLoader, path, _extensions(useDraco, useMeshOpt, extensions), 
    // @ts-expect-error - we know the type of the data
    onLoad);
};
_injectGLTF.setDecoderPath = (path) => {
    decoderPath = path;
};
const injectGLTF = _injectGLTF;

function injectProgress(injector) {
    return assertInjector(injectProgress, injector, () => {
        const cdr = inject(ChangeDetectorRef);
        const progress = signal({ errors: [], active: false, progress: 0, item: '', loaded: 0, total: 0 });
        let saveLastTotalLoaded = 0;
        DefaultLoadingManager.onStart = (item, loaded, total) => {
            progress.update((prev) => ({
                ...prev,
                active: true,
                item,
                loaded,
                total,
                progress: ((loaded - saveLastTotalLoaded) / (total - saveLastTotalLoaded)) * 100,
            }));
            cdr.detectChanges();
        };
        DefaultLoadingManager.onLoad = () => {
            progress.update((prev) => ({ ...prev, active: false }));
            cdr.detectChanges();
        };
        DefaultLoadingManager.onError = (url) => {
            progress.update((prev) => ({ ...prev, errors: [...prev.errors, url] }));
            cdr.detectChanges();
        };
        DefaultLoadingManager.onProgress = (item, loaded, total) => {
            if (loaded === total)
                saveLastTotalLoaded = total;
            progress.update((prev) => ({
                ...prev,
                item,
                loaded,
                total,
                progress: ((loaded - saveLastTotalLoaded) / (total - saveLastTotalLoaded)) * 100 || 100,
            }));
            cdr.detectChanges();
        };
        return progress.asReadonly();
    });
}

const defaultDataInterpolation = (p) => `Loading ${p.toFixed(2)}%`;
const defaultOptions = {
    containerClass: '',
    innerClass: '',
    barClass: '',
    dataClass: '',
    dataInterpolation: defaultDataInterpolation,
    initialState: (value) => value,
};
class NgtsLoader {
    constructor() {
        this.progressState = injectProgress();
        this.active = computed(() => this.progressState().active);
        this.progress = computed(() => this.progressState().progress);
        this.options = input(defaultOptions, { transform: mergeInputs(defaultOptions) });
        this.containerClass = pick(this.options, 'containerClass');
        this.innerClass = pick(this.options, 'innerClass');
        this.barClass = pick(this.options, 'barClass');
        this.dataClass = pick(this.options, 'dataClass');
        this.initialState = pick(this.options, 'initialState');
        this.dataInterpolation = pick(this.options, 'dataInterpolation');
        this.progressSpanRef = viewChild('progressSpanRef');
        this.shown = signal(this.initialState()(this.active()));
        effect((onCleanup) => {
            const [active, lastShown] = [this.active(), untracked(this.shown)];
            if (lastShown !== active) {
                const timeoutId = setTimeout(() => {
                    this.shown.set(active);
                }, 300);
                onCleanup(() => clearTimeout(timeoutId));
            }
        });
        let progressRef = 0;
        let rafId;
        effect((onCleanup) => {
            const [dataInterpolation, progress] = [this.dataInterpolation(), this.progress()];
            const updateProgress = () => {
                const progressSpan = this.progressSpanRef()?.nativeElement;
                if (!progressSpan)
                    return;
                progressRef += (progress - progressRef) / 2;
                if (progressRef > 0.95 * progress || progress === 100)
                    progressRef = progress;
                progressSpan.innerText = dataInterpolation(progressRef);
                if (progressRef < progress) {
                    rafId = requestAnimationFrame(updateProgress);
                }
            };
            updateProgress();
            onCleanup(() => cancelAnimationFrame(rafId));
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLoader, deps: [], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "19.1.4", type: NgtsLoader, isStandalone: true, selector: "ngts-loader", inputs: { options: { classPropertyName: "options", publicName: "options", isSignal: true, isRequired: false, transformFunction: null } }, viewQueries: [{ propertyName: "progressSpanRef", first: true, predicate: ["progressSpanRef"], descendants: true, isSignal: true }], ngImport: i0, template: `
		@if (shown()) {
			<div
				class="ngts-loader-container"
				[class]="containerClass() || ''"
				[style.--ngts-loader-container-opacity]="active() ? 1 : 0"
			>
				<div>
					<div class="ngts-loader-inner" [class]="innerClass() || ''">
						<div
							class="ngts-loader-bar"
							[class]="barClass() || ''"
							[style.--ngts-loader-bar-scale]="progress() / 100"
						></div>
						<span #progressSpanRef class="ngts-loader-data" [class]="dataClass() || ''"></span>
					</div>
				</div>
			</div>
		}
	`, isInline: true, styles: [".ngts-loader-container{--ngts-loader-container-opacity: 0;position:absolute;top:0;left:0;width:100%;height:100%;background:#171717;display:flex;align-items:center;justify-content:center;transition:opacity .3s ease;z-index:1000;opacity:var(--ngts-loader-container-opacity)}.ngts-loader-inner{width:100px;height:3px;background:#272727;text-align:center}.ngts-loader-bar{--ngts-loader-bar-scale: 0;height:3px;width:100px;background:#fff;transition:transform .2s;transform-origin:left center;transform:scaleX(var(--ngts-loader-bar-scale))}.ngts-loader-data{display:inline-block;position:relative;font-variant-numeric:tabular-nums;margin-top:.8em;color:#f0f0f0;font-size:.6em;font-family:-apple-system,BlinkMacSystemFont,Inter,Segoe UI,Helvetica Neue,Helvetica,Arial,Roboto,Ubuntu,sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",Segoe UI Symbol;white-space:nowrap}\n"], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "19.1.4", ngImport: i0, type: NgtsLoader, decorators: [{
            type: Component,
            args: [{ selector: 'ngts-loader', template: `
		@if (shown()) {
			<div
				class="ngts-loader-container"
				[class]="containerClass() || ''"
				[style.--ngts-loader-container-opacity]="active() ? 1 : 0"
			>
				<div>
					<div class="ngts-loader-inner" [class]="innerClass() || ''">
						<div
							class="ngts-loader-bar"
							[class]="barClass() || ''"
							[style.--ngts-loader-bar-scale]="progress() / 100"
						></div>
						<span #progressSpanRef class="ngts-loader-data" [class]="dataClass() || ''"></span>
					</div>
				</div>
			</div>
		}
	`, changeDetection: ChangeDetectionStrategy.OnPush, styles: [".ngts-loader-container{--ngts-loader-container-opacity: 0;position:absolute;top:0;left:0;width:100%;height:100%;background:#171717;display:flex;align-items:center;justify-content:center;transition:opacity .3s ease;z-index:1000;opacity:var(--ngts-loader-container-opacity)}.ngts-loader-inner{width:100px;height:3px;background:#272727;text-align:center}.ngts-loader-bar{--ngts-loader-bar-scale: 0;height:3px;width:100px;background:#fff;transition:transform .2s;transform-origin:left center;transform:scaleX(var(--ngts-loader-bar-scale))}.ngts-loader-data{display:inline-block;position:relative;font-variant-numeric:tabular-nums;margin-top:.8em;color:#f0f0f0;font-size:.6em;font-family:-apple-system,BlinkMacSystemFont,Inter,Segoe UI,Helvetica Neue,Helvetica,Arial,Roboto,Ubuntu,sans-serif,\"Apple Color Emoji\",\"Segoe UI Emoji\",Segoe UI Symbol;white-space:nowrap}\n"] }]
        }], ctorParameters: () => [] });

function _injectTexture(input, { onLoad, injector } = {}) {
    return assertInjector(_injectTexture, injector, () => {
        const store = injectStore();
        const result = injectLoader(() => TextureLoader, input);
        effect(() => {
            const textures = result();
            if (!textures)
                return;
            const gl = store.get('gl');
            if ('initTexture' in gl) {
                const array = Array.isArray(textures)
                    ? textures
                    : textures instanceof Texture
                        ? [textures]
                        : Object.values(textures);
                if (onLoad)
                    onLoad(array);
                array.forEach(gl.initTexture.bind(gl));
            }
        });
        return result;
    });
}
_injectTexture.preload = (input) => {
    injectLoader.preload(() => TextureLoader, input);
};
const injectTexture = _injectTexture;

/**
 * Generated bundle index. Do not edit.
 */

export { NgtsLoader, injectFBX, injectFont, injectGLTF, injectProgress, injectTexture };
//# sourceMappingURL=angular-three-soba-loaders.mjs.map
